@@ -1,4 +1,4 @@
-/* Copyright 2007 Matthew Barnes
+/* Copyright 2007, 2008 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -41,7 +41,10 @@
 #define SQL_CREATE_TABLE_GAME \
         "CREATE TABLE IF NOT EXISTS game (" \
                 "name PRIMARY KEY, " \
+                "category, " \
                 "sourcefile, " \
+                "isbios DEFAULT 'no' " \
+                "CHECK (isbios in ('yes', 'no')), " \
                 "runnable DEFAULT 'yes' " \
                 "CHECK (runnable in ('yes', 'no')), " \
                 "cloneof, " \
@@ -144,14 +147,21 @@
         "CREATE TABLE IF NOT EXISTS display (" \
                 "game NOT NULL, " \
                 "type NOT NULL " \
-                "CHECK (type in ('raster', 'vector')), " \
+                "CHECK (type in ('raster', 'vector', 'lcd', 'unknown')), " \
                 "rotate NOT NULL " \
                 "CHECK (rotate in ('0', '90', '180', '270')), " \
                 "flipx DEFAULT 'no' " \
                 "CHECK (flipx in ('yes', 'no')), " \
                 "width, " \
                 "height, " \
-                "refresh NOT NULL);"
+                "refresh NOT NULL, " \
+                "pixclock, " \
+                "htotal, " \
+                "hbend, " \
+                "hbstart, " \
+                "vtotal, " \
+                "vbend, " \
+                "vbstart);"
 
 #define SQL_CREATE_TABLE_CONTROL \
         "CREATE TABLE IF NOT EXISTS control (" \
@@ -172,16 +182,41 @@
                 "default_ DEFAULT 'no' " \
                 "CHECK (default_ in ('yes', 'no')));"
 
+/* The playback table survives database builds. */
+#define SQL_CREATE_TABLE_PLAYBACK \
+        "CREATE TABLE IF NOT EXISTS playback (" \
+                "name NOT NULL, " \
+                "inode NOT NULL, " \
+                "comment);"
+
 #define SQL_CREATE_VIEW_AVAILABLE \
         "CREATE VIEW IF NOT EXISTS available AS " \
-                "SELECT *, getcategory(name) AS category, " \
-                "isfavorite(name) AS favorite FROM game " \
-                "WHERE (romset IN ('good', 'best available'))"
+                "SELECT game.*, bios.description AS bios, " \
+                "isfavorite(game.name) AS favorite FROM game " \
+                "LEFT JOIN (SELECT name, description FROM game WHERE " \
+                "isbios = 'yes') AS bios ON game.romof = bios.name " \
+                "WHERE (romset IN ('good', 'best available') " \
+                "AND isbios = 'no');"
+
+#define SQL_DROP_TABLES \
+        "DROP TABLE IF EXISTS mame; " \
+        "DROP TABLE IF EXISTS game; " \
+        "DROP TABLE IF EXISTS biosset; " \
+        "DROP TABLE IF EXISTS rom; " \
+        "DROP TABLE IF EXISTS disk; " \
+        "DROP TABLE IF EXISTS sample; " \
+        "DROP TABLE IF EXISTS chip; " \
+        "DROP TABLE IF EXISTS display; " \
+        "DROP TABLE IF EXISTS control; " \
+        "DROP TABLE IF EXISTS dipvalue; " \
+        "DROP VIEW IF EXISTS available"
 
 #define SQL_INSERT_GAME \
         "INSERT INTO game VALUES (" \
                 "@name, " \
+                "@category, " \
                 "@sourcefile, " \
+                "@isbios, " \
                 "@runnable, " \
                 "@cloneof, " \
                 "@romof, " \
@@ -260,7 +295,14 @@
                 "@flipx, " \
                 "@width, " \
                 "@height, " \
-                "@refresh);"
+                "@refresh, " \
+                "@pixclock, " \
+                "@htotal, " \
+                "@hbend, " \
+                "@hbstart, " \
+                "@vtotal, " \
+                "@vbend, " \
+                "@vbstart);"
 
 #define SQL_INSERT_CONTROL \
         "INSERT INTO control VALUES (" \
@@ -332,9 +374,13 @@ static struct
         const gchar *flipx;
         const gchar *game;
         const gchar *graphic;
+        const gchar *hbend;
+        const gchar *hbstart;
         const gchar *height;
+        const gchar *htotal;
         const gchar *index_;
         const gchar *input;
+        const gchar *isbios;
         const gchar *keydelta;
         const gchar *mame;
         const gchar *manufacturer;
@@ -346,6 +392,7 @@ static struct
         const gchar *offset;
         const gchar *orientation;
         const gchar *palettesize;
+        const gchar *pixclock;
         const gchar *players;
         const gchar *protection;
         const gchar *refresh;
@@ -368,6 +415,9 @@ static struct
         const gchar *status;
         const gchar *tilt;
         const gchar *type;
+        const gchar *vbend;
+        const gchar *vbstart;
+        const gchar *vtotal;
         const gchar *width;
         const gchar *year;
 
@@ -380,16 +430,18 @@ db_parser_bind_text (sqlite3_stmt *stmt,
                      const gchar *param,
                      const gchar *value)
 {
+        gint index;
         gint errcode;
+        gchar *utf8;
         GError *error = NULL;
 
-        errcode = sqlite3_bind_text (
-                stmt, sqlite3_bind_parameter_index (stmt, param),
-                g_locale_to_utf8 (value, -1, NULL, NULL, &error), -1, g_free);
-
-        /* Handle conversion errors. */
+        index = sqlite3_bind_parameter_index (stmt, param);
+        utf8 = g_locale_to_utf8 (value, -1, NULL, NULL, &error);
         gva_error_handle (&error);
 
+        g_return_if_fail (utf8 != NULL);
+        errcode = sqlite3_bind_text (stmt, index, utf8, -1, g_free);
+
         if (errcode != SQLITE_OK)
         {
                 gva_db_set_error (&error, 0, NULL);
@@ -621,6 +673,20 @@ db_parser_start_element_display (ParserData *data,
                         param = "@height";
                 else if (attribute_name[ii] == intern.refresh)
                         param = "@refresh";
+                else if (attribute_name[ii] == intern.pixclock)
+                        param = "@pixclock";
+                else if (attribute_name[ii] == intern.htotal)
+                        param = "@htotal";
+                else if (attribute_name[ii] == intern.hbend)
+                        param = "@hbend";
+                else if (attribute_name[ii] == intern.hbstart)
+                        param = "@hbstart";
+                else if (attribute_name[ii] == intern.vtotal)
+                        param = "@vtotal";
+                else if (attribute_name[ii] == intern.vbend)
+                        param = "@vbend";
+                else if (attribute_name[ii] == intern.vbstart)
+                        param = "@vbstart";
                 else
                         continue;
 
@@ -673,9 +739,13 @@ db_parser_start_element_game (ParserData *data,
                               GError **error)
 {
         sqlite3_stmt *stmt = data->insert_game_stmt;
+        gchar *category;
+        gint error_code;
         gint ii;
+        GError *local_error = NULL;
 
         /* Bind default values. */
+        db_parser_bind_text (stmt, "@isbios", "no");
         db_parser_bind_text (stmt, "@runnable", "yes");
 
         for (ii = 0; attribute_name[ii] != NULL; ii++)
@@ -689,6 +759,8 @@ db_parser_start_element_game (ParserData *data,
                 }
                 else if (attribute_name[ii] == intern.sourcefile)
                         param = "@sourcefile";
+                else if (attribute_name[ii] == intern.isbios)
+                        param = "@isbios";
                 else if (attribute_name[ii] == intern.runnable)
                         param = "@runnable";
                 else if (attribute_name[ii] == intern.cloneof)
@@ -702,6 +774,24 @@ db_parser_start_element_game (ParserData *data,
 
                 db_parser_bind_text (stmt, param, attribute_value[ii]);
         }
+
+#ifdef CATEGORY_FILE
+        /* Lookup category from the catver.ini file. */
+        g_return_if_fail (data->game != NULL);
+        category = gva_categories_lookup (data->game, &local_error);
+
+        /* Slience "key not found" errors. */
+        error_code = G_KEY_FILE_ERROR_KEY_NOT_FOUND;
+        if (g_error_matches (local_error, G_KEY_FILE_ERROR, error_code))
+                g_clear_error (&local_error);
+
+        if (category != NULL)
+                db_parser_bind_text (stmt, "@category", category);
+        else if (local_error != NULL)
+                g_propagate_error (error, local_error);
+
+        g_free (category);
+#endif
 }
 
 static void
@@ -882,7 +972,15 @@ db_parser_start_element (GMarkupParseContext *context,
 
         /* XXX Copied from below... */
 
-        if (element_name == intern.driver)
+        if (element_name == intern.chip)
+                db_parser_start_element_chip (
+                        data, attribute_name, attribute_value, error);
+
+        else if (element_name == intern.display)
+                db_parser_start_element_display (
+                        data, attribute_name, attribute_value, error);
+
+        else if (element_name == intern.driver)
                 db_parser_start_element_driver (
                         data, attribute_name, attribute_value, error);
 
@@ -993,7 +1091,13 @@ db_parser_end_element (GMarkupParseContext *context,
 
         /* XXX Copied from below... */
 
-        if (element_name == intern.game)
+        if (element_name == intern.chip)
+                db_parser_exec_stmt (data->insert_chip_stmt, error);
+
+        else if (element_name == intern.display)
+                db_parser_exec_stmt (data->insert_display_stmt, error);
+
+        else if (element_name == intern.game)
                 db_parser_end_element_game (data, error);
 
         /* Skip unused elements to speed up parsing. */
@@ -1191,36 +1295,10 @@ db_create_tables (GError **error)
                 && gva_db_execute (SQL_CREATE_TABLE_DISPLAY, error)
                 && gva_db_execute (SQL_CREATE_TABLE_CONTROL, error)
                 && gva_db_execute (SQL_CREATE_TABLE_DIPVALUE, error)
+                && gva_db_execute (SQL_CREATE_TABLE_PLAYBACK, error)
                 && gva_db_execute (SQL_CREATE_VIEW_AVAILABLE, error);
 }
 
-static void
-db_function_getcategory (sqlite3_context *context,
-                         gint n_values,
-                         sqlite3_value **values)
-{
-#ifdef CATEGORY_FILE
-        const gchar *name;
-        gchar *category;
-        GError *error = NULL;
-
-        g_assert (n_values == 1);
-
-        name = (const gchar *) sqlite3_value_text (values[0]);
-        category = gva_categories_lookup (name, &error);
-	gva_error_handle (&error);
-
-        if (category != NULL)
-                sqlite3_result_text (context, category, -1, SQLITE_TRANSIENT);
-	else
-		sqlite3_result_null (context);
-
-	g_free (category);
-#else
-	sqlite3_result_null (context);
-#endif
-}
-
 static void
 db_function_isfavorite (sqlite3_context *context,
                         gint n_values,
@@ -1237,6 +1315,39 @@ db_function_isfavorite (sqlite3_context *context,
                 sqlite3_result_text (context, "no", -1, SQLITE_STATIC);
 }
 
+static void
+db_function_match (sqlite3_context *context,
+                   gint n_values,
+                   sqlite3_value **values)
+{
+        const gchar *text1;
+        const gchar *text2;
+        gboolean match = FALSE;
+
+        g_assert (n_values == 2);
+
+        /* XXX Note the operands are not in the order one might think.
+         *     Still trying to find some documentation to confirm this. */
+
+        text1 = (const gchar *) sqlite3_value_text (values[1]);
+        text2 = (const gchar *) sqlite3_value_text (values[0]);
+
+        if (text1 != NULL && text2 != NULL)
+        {
+                gchar *s1, *s2;
+
+                s1 = gva_search_collate_key (text1);
+                s2 = gva_search_collate_key (text2);
+
+                match = (strstr (s1, s2) != NULL);
+
+                g_free (s1);
+                g_free (s2);
+        }
+
+        sqlite3_result_int (context, match);
+}
+
 static void
 db_trace_cb (gpointer unused, const gchar *message)
 {
@@ -1277,8 +1388,8 @@ gva_db_init (GError **error)
                 goto fail;
 
         errcode = sqlite3_create_function (
-                db, "getcategory", 1, SQLITE_ANY, NULL,
-                db_function_getcategory, NULL, NULL);
+                db, "match", 2, SQLITE_ANY, NULL,
+                db_function_match, NULL, NULL);
         if (errcode != SQLITE_OK)
                 goto fail;
 
@@ -1339,9 +1450,13 @@ gva_db_build (GError **error)
         intern.flipx        = g_intern_static_string ("flipx");
         intern.game         = g_intern_static_string ("game");
         intern.graphic      = g_intern_static_string ("graphic");
+        intern.hbend        = g_intern_static_string ("hbend");
+        intern.hbstart      = g_intern_static_string ("hbstart");
         intern.height       = g_intern_static_string ("height");
+        intern.htotal       = g_intern_static_string ("htotal");
         intern.index_       = g_intern_static_string ("index_");
         intern.input        = g_intern_static_string ("input");
+        intern.isbios       = g_intern_static_string ("isbios");
         intern.keydelta     = g_intern_static_string ("keydelta");
         intern.mame         = g_intern_static_string ("mame");
         intern.manufacturer = g_intern_static_string ("manufacturer");
@@ -1353,6 +1468,7 @@ gva_db_build (GError **error)
         intern.offset       = g_intern_static_string ("offset");
         intern.orientation  = g_intern_static_string ("orientation");
         intern.palettesize  = g_intern_static_string ("palettesize");
+        intern.pixclock     = g_intern_static_string ("pixclock");
         intern.players      = g_intern_static_string ("players");
         intern.protection   = g_intern_static_string ("protection");
         intern.refresh      = g_intern_static_string ("refresh");
@@ -1375,6 +1491,9 @@ gva_db_build (GError **error)
         intern.status       = g_intern_static_string ("status");
         intern.tilt         = g_intern_static_string ("tilt");
         intern.type         = g_intern_static_string ("type");
+        intern.vbend        = g_intern_static_string ("vbend");
+        intern.vbstart      = g_intern_static_string ("vbstart");
+        intern.vtotal       = g_intern_static_string ("vtotal");
         intern.width        = g_intern_static_string ("width");
         intern.year         = g_intern_static_string ("year");
 
@@ -1417,19 +1536,12 @@ gva_db_build (GError **error)
 gboolean
 gva_db_reset (GError **error)
 {
-        const gchar *filename;
-
         g_return_val_if_fail (db != NULL, FALSE);
 
-        filename = gva_db_get_filename ();
+        if (!gva_db_execute (SQL_DROP_TABLES, error))
+                return FALSE;
 
-        sqlite3_close (db);
-        db = NULL;
-
-        if (g_file_test (filename, G_FILE_TEST_EXISTS))
-                g_remove (filename);
-
-        return gva_db_init (error);
+        return db_create_tables (error);
 }
 
 /**
@@ -1442,7 +1554,7 @@ gva_db_reset (GError **error)
  * "BEGIN TRANSACTION".  If an error occurs, it returns %FALSE and sets
  * @error.
  *
- * Returns: %TRUE on succes, %FALSE if an error occurred
+ * Returns: %TRUE on success, %FALSE if an error occurred
  **/
 gboolean
 gva_db_execute (const gchar *sql,
@@ -1465,6 +1577,62 @@ gva_db_execute (const gchar *sql,
         return (errcode == SQLITE_OK);
 }
 
+/**
+ * gva_db_get_table:
+ * @sql: an SQL statement
+ * @result: return location for the result
+ * @rows: return location for the number of rows in the result
+ * @columns: return location for the number of columns in the result
+ * @error: return location for a #GError, or %NULL
+ *
+ * Executes the given SQL statement and returns the results as a string
+ * array (see sqlite3_get_table() for the array layout).  This function
+ * is appropriate for SELECT statements that return a small result set.
+ * If an error occurs, it returns %FALSE and sets @error.
+ *
+ * Use g_strfreev() to free @result.
+ *
+ * Returns: %TRUE on success, %FALSE if an error occurred
+ **/
+gboolean
+gva_db_get_table (const gchar *sql,
+                  gchar ***result,
+                  gint *rows,
+                  gint *columns,
+                  GError **error)
+{
+        char **table;
+        char *errmsg;
+        gint errcode;
+
+        g_return_val_if_fail (db != NULL, FALSE);
+        g_return_val_if_fail (sql != NULL, FALSE);
+        g_return_val_if_fail (result != NULL, FALSE);
+        g_return_val_if_fail (rows != NULL, FALSE);
+        g_return_val_if_fail (columns != NULL, FALSE);
+
+        errcode = sqlite3_get_table (
+                db, sql, &table, rows, columns, &errmsg);
+
+        if (errcode == SQLITE_OK)
+        {
+                gint length, ii;
+
+                length = (*rows + 1) * (*columns);
+                *result = g_new0 (gchar *, length + 1);
+                for (ii = 0; ii < length; ii++)
+                        (*result)[ii] = g_strdup (table[ii]);
+                sqlite3_free_table (table);
+        }
+        else
+        {
+                gva_db_set_error (error, errcode, errmsg);
+                sqlite3_free (errmsg);
+        }
+
+        return (errcode == SQLITE_OK);
+}
+
 /**
  * gva_db_transaction_begin:
  * @error: return location for a #GError, or %NULL
@@ -1610,6 +1778,39 @@ gva_db_get_filename (void)
         return filename;
 }
 
+/**
+ * gva_db_is_older_than:
+ * @filename: a file or directory name
+ *
+ * Returns %TRUE if @filename's creation or modification timestamp is
+ * more recent than the games database's modification timestamp.  The
+ * games database relies in part on external data files that might
+ * have been updated since the database was last rebuilt.  This
+ * function can detect that.
+ *
+ * If @filename does not exist or if the function has trouble comparing
+ * timestamps, it returns %FALSE as a safe fallback.
+ *
+ * Returns: %TRUE if @filename is newer than the games database
+ **/
+gboolean
+gva_db_is_older_than (const gchar *filename)
+{
+        time_t db_mtime;
+        struct stat st;
+
+        if (g_stat (gva_db_get_filename (), &st) < 0)
+                return FALSE;
+
+        db_mtime = st.st_mtime;
+
+        if (g_stat (filename, &st) < 0)
+                return FALSE;
+
+        return (db_mtime < st.st_mtime) || (db_mtime < st.st_ctime);
+}
+
+
 /**
  * gva_db_needs_rebuilt:
  *
@@ -1696,5 +1897,5 @@ gva_db_set_error (GError **error,
                 message = sqlite3_errmsg (db);
         }
 
-        g_set_error (error, GVA_SQLITE_ERROR, code, message);
+        g_set_error (error, GVA_SQLITE_ERROR, code, "(SQL) %s", message);
 }
