@@ -57,12 +57,16 @@ typedef struct _GvaGameStoreClass GvaGameStoreClass;
  * GvaGameStoreColumn:
  * @GVA_GAME_STORE_COLUMN_NAME:
  *      Corresponds to the "available.name" database field.
+ * @GVA_GAME_STORE_COLUMN_BIOS:
+ *      Corresponds to the "available.bios" database field.
  * @GVA_GAME_STORE_COLUMN_CATEGORY:
  *      Corresponds to the "available.category" database field.
  * @GVA_GAME_STORE_COLUMN_FAVORITE:
  *      Corresponds to the "available.favorite" database field.
  * @GVA_GAME_STORE_COLUMN_SOURCEFILE:
  *      Corresponds to the "available.sourcefile" database field.
+ * @GVA_GAME_STORE_COLUMN_ISBIOS:
+ *      Corresponds to the "available.isbios" database field.
  * @GVA_GAME_STORE_COLUMN_RUNNABLE:
  *      Corresponds to the "available.runnable" database field.
  * @GVA_GAME_STORE_COLUMN_CLONEOF:
@@ -127,9 +131,11 @@ typedef struct _GvaGameStoreClass GvaGameStoreClass;
 typedef enum
 {
         GVA_GAME_STORE_COLUMN_NAME,               /* G_TYPE_STRING */
+        GVA_GAME_STORE_COLUMN_BIOS,               /* G_TYPE_STRING */
         GVA_GAME_STORE_COLUMN_CATEGORY,           /* G_TYPE_STRING */
         GVA_GAME_STORE_COLUMN_FAVORITE,           /* G_TYPE_BOOLEAN */
         GVA_GAME_STORE_COLUMN_SOURCEFILE,         /* G_TYPE_STRING */
+        GVA_GAME_STORE_COLUMN_ISBIOS,             /* G_TYPE_BOOLEAN */
         GVA_GAME_STORE_COLUMN_RUNNABLE,           /* G_TYPE_BOOLEAN */
         GVA_GAME_STORE_COLUMN_CLONEOF,            /* G_TYPE_STRING */
         GVA_GAME_STORE_COLUMN_ROMOF,              /* G_TYPE_STRING */
