@@ -23,6 +23,12 @@
 #include "gva-error.h"
 #include "gva-mame.h"
 
+#ifdef WITH_GNOME
+#include <gnome.h>
+#endif
+
+#define DEFAULT_MONOSPACE_FONT_NAME     "Monospace 10"
+
 #define GCONF_MONOSPACE_FONT_NAME_KEY \
         "/desktop/gnome/interface/monospace_font_name"
 
@@ -39,6 +45,20 @@ inpname_exists (const gchar *inppath, const gchar *inpname)
         gchar *filename;
         gboolean exists;
 
+        /* Last version of xmame creates input files named "game.inp",
+         * but sdlmame names them "game".  Not sure when the extension
+         * was dropped (tested versions 0.120 - 0.123).  In any case,
+         * we need to check for both. */
+
+        inpfile = g_strdup_printf ("%s", inpname);
+        filename = g_build_filename (inppath, inpfile, NULL);
+        exists = g_file_test (filename, G_FILE_TEST_EXISTS);
+        g_free (filename);
+        g_free (inpfile);
+
+        if (exists)
+                return TRUE;
+
         inpfile = g_strdup_printf ("%s.inp", inpname);
         filename = g_build_filename (inppath, inpfile, NULL);
         exists = g_file_test (filename, G_FILE_TEST_EXISTS);
@@ -62,14 +82,14 @@ inpname_exists (const gchar *inppath, const gchar *inpname)
 gchar *
 gva_choose_inpname (const gchar *game)
 {
+        const gchar *inppath;
         gchar *inpname;
-        gchar *inppath;
         gint nn = 1;
         GError *error = NULL;
 
         g_return_val_if_fail (game != NULL, NULL);
 
-        inppath = gva_mame_get_config_value ("input_directory", &error);
+        inppath = gva_mame_get_input_directory (&error);
 
         if (inppath == NULL || !inpname_exists (inppath, game))
         {
@@ -84,8 +104,6 @@ gva_choose_inpname (const gchar *game)
                 g_free (inpname);
         }
 
-        g_free (inppath);
-
         return inpname;
 }
 
@@ -220,6 +238,10 @@ gva_get_monospace_font_name (void)
         gva_error_handle (&error);
         g_object_unref (client);
 
+        /* Fallback to a reasonable default. */
+        if (font_name == NULL)
+                font_name = g_strdup (DEFAULT_MONOSPACE_FONT_NAME);
+
         return font_name;
 }
 
@@ -270,3 +292,77 @@ gva_get_user_data_dir (void)
 
         return user_data_dir;
 }
+
+/**
+ * gva_help_display:
+ * @parent: a parent #GtkWindow or %NULL
+ * @link_id: help section to present or %NULL
+ *
+ * Opens the user documentation to the section given by @link_id, or to the
+ * table of contents if @link_id is %NULL.  If the user documentation cannot
+ * be opened, it presents a dialog describing the error.  The dialog is set
+ * as transient to @parent if @parent is %NULL.
+ **/
+void
+gva_help_display (GtkWindow *parent,
+                  const gchar *link_id)
+{
+#ifdef WITH_GNOME
+        GtkWidget *dialog;
+        GError *error = NULL;
+
+        if (gnome_help_display ("gnome-video-arcade.xml", link_id, &error))
+                return;
+
+        dialog = gtk_message_dialog_new_with_markup (
+                parent, GTK_DIALOG_DESTROY_WITH_PARENT,
+                GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+                "<big><b>%s</b></big>",
+                _(PACKAGE_NAME " could not display the help contents."));
+
+        gtk_message_dialog_format_secondary_text (
+                GTK_MESSAGE_DIALOG (dialog), "%s", error->message);
+
+        gtk_dialog_run (GTK_DIALOG (dialog));
+
+        gtk_widget_destroy (dialog);
+        g_error_free (error);
+#endif
+}
+
+/**
+ * gva_search_collate_key:
+ * @string: a string
+ *
+ * Converts a string into a collation key that can be compared with other
+ * collation keys produced by the same function using strcmp().
+ *
+ * Specifically, the function filters out spaces and punctuation from @string
+ * for easier comparison with what a human is likely to type in an interactive
+ * search.  e.g. Typing "mspacman" will match "Ms. Pac-Man".
+ *
+ * Returns: a newly-allocated collation key
+ **/
+gchar *
+gva_search_collate_key (const gchar *string)
+{
+        const gchar *valid_chars;
+        gchar **str_array;
+        gchar *temp;
+
+        /* FIXME This doesn't take UTF-8 into account. */
+
+        g_return_val_if_fail (string != NULL, NULL);
+
+        valid_chars = "abcdefghijklmnopqrstuvwxyz"
+                      "0123456789";
+
+        temp = g_ascii_strdown (string, -1);
+        g_strcanon (temp, valid_chars, '?');
+        str_array = g_strsplit_set (temp, "?", -1);
+        g_free (temp);
+        temp = g_strjoinv (NULL, str_array);
+        g_strfreev (str_array);
+
+        return temp;
+}
